@@ -1,21 +1,21 @@
 /*  -*-C++-*- -*-coding: utf-8-unix;-*-
-    Classified Ads is Copyright (c) Antti Jarvinen 2015.
+  Classified Ads is Copyright (c) Antti Jarvinen 2015-2022.
 
-    This file is part of Classified Ads.
+  This file is part of Classified Ads.
 
-    Classified Ads is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  Classified Ads is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Classified Ads is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+  Classified Ads is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Classified Ads; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Classified Ads; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 
@@ -26,6 +26,8 @@
 #include <locale.h>
 #ifdef WIN32
 #include <QLocale>
+#include <QCoreApplication>
+#include <QDir>
 #endif
 /**
  * Translation domain for gnu gettext library
@@ -53,7 +55,9 @@ CATranslator::CATranslator(QObject* aParent) :
     setlocale(LC_MESSAGES,languageName.toUtf8().constData());
     // in win32, assume the translation files reside under
     // same directory as the binary
-    bindtextdomain(KProgramName,"./");
+    QString applicationDirectory ( QCoreApplication::applicationDirPath() ) ;
+    applicationDirectory.append(QDir::separator()) ;
+    bindtextdomain(KProgramName, applicationDirectory.toLocal8Bit().data());
 #else
     setlocale(LC_ALL,"");
     // in unix-like systems the installer puts files here:
